import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_backdrop_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorBackdrop extends StatefulWidget {
  const SeniorBackdrop({
    Key? key,
    this.actions,
    this.backdropTabBarInfo,
    required this.body,
    this.centerTitle,
    this.hasTopPadding = true,
    this.hideLeading = false,
    this.leading,
    this.onRefresh,
    this.onTapBack,
    this.systemOverlayStyle,
    this.style,
    required this.title,
  }) : super(key: key);

  final List<Widget>? actions;
  final SeniorBackdropTabBarInfo? backdropTabBarInfo;
  final Widget body;
  final bool? centerTitle;
  final bool hasTopPadding;
  final bool hideLeading;
  final Widget? leading;
  final RefreshCallback? onRefresh;
  final Function()? onTapBack;
  final SystemUiOverlayStyle? systemOverlayStyle;
  final SeniorBackdropStyle? style;
  final Widget title;

  @override
  State<SeniorBackdrop> createState() => _SeniorBackdropState();
}

class _SeniorBackdropState extends State<SeniorBackdrop> {
  final ScrollController backdropController = ScrollController(keepScrollOffset: false);
  final appBarHeight = 56;
  final tabBarHeight = 48;
  bool hasPadding = false;
  double appBarCurrentHeight = 0.0;

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Container(
      height: MediaQuery.of(context).size.height,
      decoration: _buildBoxDecoration(theme, defaultTheme),
      child: Scaffold(
        backgroundColor: Colors.transparent,
        body: SafeArea(
          child: NotificationListener<ScrollNotification>(
            onNotification: (notify) {
              //This delayed is necessary because of the refreshIndicator used in the body.
              //When using a setState in a build process it gives a problem
              //keep the duration as zero ALWAYS.
              Future.delayed(Duration.zero, () {
                setState(() {
                  if (widget.backdropTabBarInfo == null) _updateAppBarCurrentHeight();
                  if (notify is ScrollUpdateNotification) {
                    if (widget.backdropTabBarInfo != null) _updateAppBarCurrentHeight();
                    hasPadding = _hasPadding(
                      appBarCurrentHeight,
                      backdropController.position.userScrollDirection,
                    );
                  }
                });
              });

              return true;
            },
            child: NestedScrollView(
              controller: backdropController,
              floatHeaderSlivers: true,
              physics: PageScrollPhysics(),
              headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
                return <Widget>[
                  _buildAppBar(theme, defaultTheme),
                ];
              },
              body: _buildBody(theme, defaultTheme),
            ),
          ),
        ),
      ),
    );
  }

  Widget _buildBody(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return Padding(
      padding: EdgeInsets.only(top: (hasPadding) ? _getPadding(widget.backdropTabBarInfo == null) : 0),
      child: Container(
        padding: EdgeInsets.only(
          top: widget.hasTopPadding ? SeniorRadius.huge : 0,
        ),
        decoration: BoxDecoration(
          color: widget.style?.bodyColor ??
              theme?.backdropTheme?.style?.bodyColor ??
              defaultTheme.backdropTheme?.style?.bodyColor ??
              SeniorColors.pureWhite,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(SeniorRadius.huge),
            topRight: Radius.circular(SeniorRadius.huge),
          ),
        ),
        child: RefreshIndicator(
          onRefresh: widget.onRefresh != null ? widget.onRefresh! : () async {},
          child: widget.body,
        ),
      ),
    );
  }

  SliverAppBar _buildAppBar(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return SliverAppBar(
      pinned: true,
      primary: true,
      floating: true,
      elevation: 0,
      backgroundColor: Colors.transparent,
      shadowColor: Colors.transparent,
      actions: widget.actions,
      title: widget.title,
      centerTitle: widget.centerTitle,
      bottom: widget.backdropTabBarInfo != null ? _buildTabBar(theme, defaultTheme) : null,
      systemOverlayStyle: widget.systemOverlayStyle,
      leadingWidth: widget.hideLeading ? 0 : null,
      leading: widget.hideLeading
          ? SizedBox.shrink()
          : IconButton(
              icon: const Icon(FontAwesomeIcons.angleLeft),
              onPressed: widget.onTapBack ??
                  () {
                    Navigator.pop(context);
                  },
            ),
    );
  }

  TabBar _buildTabBar(SeniorTheme? theme, SeniorTheme defaultTheme) {
    final Color labelColor = widget.style?.selectedLabelColor ??
        theme?.backdropTheme?.style?.selectedLabelColor ??
        defaultTheme.backdropTheme?.style?.selectedLabelColor ??
        SeniorColors.pureWhite;

    final Color unselectedLabelColor = widget.style?.unselectedLabelColor ??
        theme?.backdropTheme?.style?.unselectedLabelColor ??
        defaultTheme.backdropTheme?.style?.unselectedLabelColor ??
        SeniorColors.pureWhite.withOpacity(.6);

    final Color indicatorColor = widget.style?.tabIndicatorColor ??
        theme?.backdropTheme?.style?.tabIndicatorColor ??
        defaultTheme.backdropTheme?.style?.tabIndicatorColor ??
        SeniorColors.pureWhite;

    return TabBar(
      tabs: widget.backdropTabBarInfo!.tabs.map((tab) => Tab(text: tab)).toList(),
      labelColor: labelColor,
      unselectedLabelColor: unselectedLabelColor,
      labelStyle: SeniorTypography.bodySemiBold(color: labelColor),
      indicator: SeniorTabIndicator(color: indicatorColor),
      physics: BouncingScrollPhysics(),
      isScrollable: widget.backdropTabBarInfo!.isScrollable,
      onTap: widget.backdropTabBarInfo!.onTap,
      controller: widget.backdropTabBarInfo!.controller,
    );
  }

  BoxDecoration _buildBoxDecoration(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return BoxDecoration(
      gradient: LinearGradient(
        colors: widget.style?.headerColors ??
            theme?.backdropTheme?.style?.headerColors ??
            defaultTheme.backdropTheme?.style?.headerColors ??
            SeniorColors.primaryGradientColors,
      ),
    );
  }

  double _getPadding(bool noHasBottomBar) =>
      noHasBottomBar ? (appBarHeight - appBarCurrentHeight) : (tabBarHeight - appBarCurrentHeight);

  bool _hasPadding(double appBarScrollPosition, ScrollDirection scrollDirection) => (widget.backdropTabBarInfo == null)
      ? (appBarScrollPosition < appBarHeight)
      : (appBarScrollPosition < tabBarHeight);

  double _updateAppBarCurrentHeight() => appBarCurrentHeight = backdropController.position.extentAfter;
}

class SeniorTabIndicator extends Decoration {
  final BoxPainter _painter;
  final Color color;

  SeniorTabIndicator({required this.color}) : _painter = _SeniorPainter(color);

  @override
  BoxPainter createBoxPainter([VoidCallback? onChanged]) => _painter;
}

class _SeniorPainter extends BoxPainter {
  final Paint _paint;

  _SeniorPainter(Color color)
      : _paint = Paint()
          ..color = color
          ..isAntiAlias = true;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration cfg) {
    final Rect rect = Offset(offset.dx + cfg.size!.width / 2 - 12, (cfg.size!.height - 10)) & Size(24, 2);
    canvas.drawRRect(
      RRect.fromRectAndCorners(
        rect,
        bottomLeft: Radius.circular(1),
        bottomRight: Radius.circular(1),
        topLeft: Radius.circular(1),
        topRight: Radius.circular(1),
      ),
      _paint,
    );
  }
}

class SeniorBackdropTabBarInfo {
  final bool isScrollable;
  final Function(int) onTap;
  final List<String> tabs;
  final int initialIndex;
  final TabController controller;

  SeniorBackdropTabBarInfo({
    this.isScrollable = false,
    required this.onTap,
    required this.tabs,
    required this.initialIndex,
    required this.controller,
  });
}
