import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../components/senior_elevated_element/senior_elevated_element.dart';
import '../../components/senior_checkbox/senior_checkbox.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorCard extends StatelessWidget {
  final Widget child;
  final Function()? pushAction;
  final Color? severityColor;
  final DismissibleConfig? dismissibleConfig;

  const SeniorCard({
    Key? key,
    required this.child,
    this.pushAction,
    this.severityColor,
    this.dismissibleConfig,
  }) : super(key: key);

  factory SeniorCard.checkbox({
    Key key,
    Color? severityColor,
    DismissibleConfig? dismissibleConfig,
    Function() pushAction,
    required Widget child,
    required bool value,
    required Function(bool?) onCheck,
  }) = _SeniorCardCheckbox;

  Widget _buildCardContent({
    required Color iconColor,
    required double iconSize,
  }) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        child,
        Visibility(
          visible: pushAction != null,
          child: Icon(
            FontAwesomeIcons.chevronRight,
            color: iconColor,
            size: iconSize,
          ),
        ),
      ],
    );
  }

  Widget _buildCard({
    required Color color,
    required Color iconColor,
    required double iconSize,
    required double borderRadius,
  }) {
    return SeniorElevatedElement(
      borderRadius: borderRadius,
      elevation: SeniorElevations.dp01,
      child: Material(
        color: Colors.transparent,
        child: InkWell(
          borderRadius: BorderRadius.circular(borderRadius),
          onTap: pushAction,
          child: Ink(
            padding: EdgeInsets.only(left: SeniorSpacing.small),
            decoration: BoxDecoration(
              color: severityColor ?? color,
              borderRadius: BorderRadius.circular(borderRadius),
            ),
            child: Ink(
              padding: const EdgeInsets.only(
                top: SeniorSpacing.normal,
                bottom: SeniorSpacing.normal,
                left: SeniorSpacing.xsmall,
                right: SeniorSpacing.normal,
              ),
              decoration: BoxDecoration(
                borderRadius: BorderRadius.only(
                  topRight: Radius.circular(borderRadius),
                  bottomRight: Radius.circular(borderRadius),
                ),
                color: color,
              ),
              child: _buildCardContent(
                iconColor: iconColor,
                iconSize: iconSize,
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    final Color color = theme?.cardTheme?.style?.backgroundColor ??
        defaultTheme.cardTheme?.style?.backgroundColor ??
        SeniorColors.neutralColor100;

    final Color iconColor = theme?.cardTheme?.style?.iconColor ??
        defaultTheme.cardTheme?.style?.iconColor ??
        SeniorColors.neutralColor600;

    final double iconSize = theme?.cardTheme?.iconSize ??
        defaultTheme.cardTheme?.iconSize ??
        SeniorTypographyIconSize.body;

    final double borderRadius = theme?.cardTheme?.borderRadius ??
        defaultTheme.cardTheme?.borderRadius ??
        SeniorRadius.xbig;

    return (dismissibleConfig != null)
        ? Dismissible(
            key: dismissibleConfig!.key,
            onDismissed: (direction) {
              if (direction == DismissDirection.startToEnd) {
                dismissibleConfig!.primaryAction.onDismissed();
              } else if (direction == DismissDirection.endToStart) {
                dismissibleConfig!.secondaryAction != null
                    ? dismissibleConfig!.secondaryAction!.onDismissed()
                    : dismissibleConfig!.primaryAction.onDismissed();
              }
            },
            direction:
                dismissibleConfig!.secondaryAction != null ? DismissDirection.horizontal : DismissDirection.startToEnd,
            background: Container(
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(borderRadius),
                color: dismissibleConfig!.primaryAction.backgroundColor,
              ),
              child: Padding(
                padding: EdgeInsets.symmetric(
                  horizontal: SeniorSpacing.medium,
                ),
                child: Align(
                  alignment: Alignment.centerLeft,
                  child: FaIcon(
                    dismissibleConfig!.primaryAction.icon,
                    color: SeniorColors.pureWhite,
                    size: SeniorTypographyIconSize.h4,
                  ),
                ),
              ),
            ),
            secondaryBackground: dismissibleConfig!.secondaryAction != null
                ? Container(
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(borderRadius),
                      color: dismissibleConfig!.secondaryAction!.backgroundColor,
                    ),
                    child: Padding(
                      padding: EdgeInsets.symmetric(
                        horizontal: SeniorSpacing.medium,
                      ),
                      child: Align(
                        alignment: Alignment.centerRight,
                        child: FaIcon(
                          dismissibleConfig!.secondaryAction!.icon,
                          color: SeniorColors.pureWhite,
                          size: SeniorTypographyIconSize.h4,
                        ),
                      ),
                    ),
                  )
                : null,
            child: _buildCard(
              color: color,
              iconColor: iconColor,
              iconSize: iconSize,
              borderRadius: borderRadius,
            ),
          )
        : _buildCard(
            color: color,
            iconColor: iconColor,
            iconSize: iconSize,
            borderRadius: borderRadius,
          );
  }
}

class _SeniorCardCheckbox extends SeniorCard {
  _SeniorCardCheckbox({
    Key? key,
    Color? severityColor,
    DismissibleConfig? dismissibleConfig,
    Function()? pushAction,
    required Widget child,
    required this.value,
    required this.onCheck,
  }) : super(
          key: key,
          child: child,
          pushAction: pushAction,
          severityColor: severityColor,
          dismissibleConfig: dismissibleConfig,
        );

  final bool value;
  final Function(bool?) onCheck;

  @override
  Widget _buildCardContent({
    required Color iconColor,
    required double iconSize,
  }) {
    return Container(
      child: Stack(
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              child,
              Visibility(
                visible: pushAction != null,
                child: Icon(
                  FontAwesomeIcons.chevronRight,
                  color: iconColor,
                  size: iconSize,
                ),
              ),
            ],
          ),
          Align(
            alignment: Alignment.topRight,
            child: Container(
              width: SeniorCheckbox.size,
              height: SeniorCheckbox.size,
              child: SeniorCheckbox(
                value: value,
                onChanged: onCheck,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class DismissibleAction {
  final Color backgroundColor;
  final IconData icon;
  final Function() onDismissed;

  const DismissibleAction({
    required this.backgroundColor,
    required this.icon,
    required this.onDismissed,
  });
}

class DismissibleConfig {
  final Key key;
  final DismissibleAction primaryAction;
  final DismissibleAction? secondaryAction;

  const DismissibleConfig({
    required this.key,
    required this.primaryAction,
    this.secondaryAction,
  });
}
