import 'package:flutter/material.dart';
import 'package:senior_design_system/senior_design_system.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

OverlayEntry? _previousEntry;

void showTopSnackBar({
  required BuildContext context,
  required Widget child,
  OverlayState? overlayState,
}) async {
  overlayState ??= Overlay.of(context);
  late OverlayEntry overlayEntry;
  overlayEntry = OverlayEntry(
    builder: (_) {
      return _SeniorTopSnackbar(
        child: child,
        onDismissed: () {
          overlayEntry.remove();
          _previousEntry = null;
        },
      );
    },
  );

  _previousEntry?.remove();
  overlayState?.insert(overlayEntry);
  _previousEntry = overlayEntry;
}


class _SeniorTopSnackbar extends StatefulWidget {
  final Widget child;
  final VoidCallback onDismissed;
  final VoidCallback? onTap;

  _SeniorTopSnackbar({
    Key? key,
    required this.child,
    required this.onDismissed,
    this.onTap,
  }) : super(key: key);

  @override
  _SeniorTopSnackbarState createState() => _SeniorTopSnackbarState();
}

class _SeniorTopSnackbarState extends State<_SeniorTopSnackbar>
    with SingleTickerProviderStateMixin {
  late Animation offsetAnimation;
  late AnimationController animationController;
  double? topPosition;

  @override
  void initState() {
    topPosition = SeniorSpacing.xsmall;
    _setupAndStartAnimation();
    super.initState();
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  void _setupAndStartAnimation() async {
    animationController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 500),
      reverseDuration: Duration(milliseconds: 500),
    );

    Tween<Offset> offsetTween = Tween<Offset>(
      begin: Offset(0.0, -1.0),
      end: Offset(0.0, 0.0),
    );

    offsetAnimation = offsetTween.animate(
      CurvedAnimation(
        parent: animationController,
        curve: Curves.ease,
        reverseCurve: Curves.ease,
      ),
    )..addStatusListener((status) async {
        if (status == AnimationStatus.completed) {
          await Future.delayed(
            Duration(milliseconds: 4000),
          );
          if (mounted) {
            animationController.reverse();
            setState(() {
              topPosition = 0;
            });
          }
        }

        if (status == AnimationStatus.dismissed) {
          widget.onDismissed.call();
        }
      });

    if (mounted) {
      animationController.forward();
    }
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedPositioned(
      duration: Duration(milliseconds: 500),
      curve: Curves.ease,
      top: topPosition,
      left: SeniorSpacing.normal,
      right: SeniorSpacing.normal,
      child: SlideTransition(
        position: offsetAnimation as Animation<Offset>,
        child: SafeArea(
          child: GestureDetector(
            onTap: () {
              if (mounted) {
                widget.onTap?.call();
                animationController.reverse();
              }
            },
            child: widget.child,
          ),
        ),
      ),
    );
  }
}

class SeniorNotificationSnackBarAction {
  final String title;
  final VoidCallback? onPressed;

  SeniorNotificationSnackBarAction({
    required this.title,
    this.onPressed,
  });
}

class SeniorNotificationSnackbar extends StatefulWidget {
  final String title;
  final String message;
  final Widget? icon;
  final SeniorNotificationSnackBarAction? seniorNotificationSnackBarAction;

  SeniorNotificationSnackbar.avatar({
    Key? key,
    required this.title,
    required this.message,
    required ImageProvider imageProvider,
    required String employeeName,
    this.seniorNotificationSnackBarAction,
  }) : icon = SeniorProfilePicture(
          radius: SeniorRadius.huge,
          name: employeeName,
          imageProvider: imageProvider,
        );

  SeniorNotificationSnackbar.icon({
    Key? key,
    required this.title,
    required this.message,
    required IconData iconData,
    required Color circleColor,
    this.seniorNotificationSnackBarAction,
  }) : icon = CircleAvatar(
          radius: SeniorRadius.huge,
          backgroundColor: circleColor,
          child: Icon(
            iconData,
            size: 12,
            color: Colors.white,
          ),
        );

  SeniorNotificationSnackbar({
    Key? key,
    required this.title,
    required this.message,
    this.seniorNotificationSnackBarAction,
    this.icon,
  });

  @override
  _SeniorNotificationSnackbarState createState() =>
      _SeniorNotificationSnackbarState();
}

class _SeniorNotificationSnackbarState
    extends State<SeniorNotificationSnackbar> {
  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: widget.seniorNotificationSnackBarAction?.onPressed,
      child: Container(
        clipBehavior: Clip.hardEdge,
        decoration: BoxDecoration(
          color: SeniorColors.neutralColor200.withOpacity(0.93),
          borderRadius: kDefaultBorderRadius,
          boxShadow: kDefaultBoxShadow,
        ),
        width: double.infinity,
        padding: EdgeInsets.only(
          left: SeniorSpacing.normal,
          right: SeniorSpacing.xxsmall,
          top: SeniorSpacing.xsmall,
          bottom: SeniorSpacing.xsmall,
        ),
        child: Material(
          color: Colors.transparent,
          child: Center(
            child: Row(
              children: [
                widget.icon == null
                    ? const SizedBox.shrink()
                    : Padding(
                        padding: EdgeInsets.only(
                          right: SeniorSpacing.xsmall,
                        ),
                        child: widget.icon,
                      ),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: [
                      Text(
                        widget.title,
                        style: SeniorTypography.smallBold(
                          color: SeniorColors.secondaryColor900,
                        ),
                        textAlign: TextAlign.start,
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                        textScaleFactor: 1,
                      ),
                      Text(
                        widget.message,
                        style: SeniorTypography.small(
                          color: SeniorColors.secondaryColor900,
                        ),
                        textAlign: TextAlign.start,
                        overflow: TextOverflow.ellipsis,
                        maxLines: 3,
                        textScaleFactor: 1,
                      ),
                    ],
                  ),
                ),
                widget.seniorNotificationSnackBarAction == null
                    ? const SizedBox.shrink()
                    : Padding(
                        padding: const EdgeInsets.only(
                          left: SeniorSpacing.xsmall,
                        ),
                        child: TextButton(
                          style: ButtonStyle(
                            overlayColor:
                                MaterialStateProperty.resolveWith<Color?>(
                              (_) {
                                return SeniorColors.secondaryColor300;
                              },
                            ),
                          ),
                          onPressed: widget
                              .seniorNotificationSnackBarAction!.onPressed,
                          child: Text(
                            widget.seniorNotificationSnackBarAction!.title,
                            style: SeniorTypography.smallBold(
                              color: SeniorColors.secondaryColor900,
                            ),
                          ),
                        ),
                      ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}

const kDefaultBoxShadow = const [
  BoxShadow(
    color: Colors.black26,
    offset: Offset(0.0, 1.0),
    spreadRadius: 0.2,
    blurRadius: 4,
  ),
];

const kDefaultBorderRadius = const BorderRadius.all(
  Radius.circular(
    SeniorRadius.xsmall,
  ),
);
