import 'package:dotted_border/dotted_border.dart';
import 'package:flutter/material.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../components/senior_checkbox/senior_checkbox.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

enum SeniorSquareButtonsMenuItemType {
  primary,
  secondary,
  ghost,
}

class SeniorSquareButtonsMenuItemData {
  SeniorSquareButtonsMenuItemData({
    this.key,
    this.disabled = false,
    required this.icon,
    required this.onTap,
    required this.text,
    required this.type,
  }) {
    this.isChecked = false;
    this.onCheck = (_) {};
    this.withCheckbox = false;
  }

  SeniorSquareButtonsMenuItemData.checkbox({
    this.key,
    this.disabled = false,
    required this.icon,
    required this.isChecked,
    required this.onCheck,
    required this.onTap,
    required this.text,
    required this.type,
  }) {
    this.withCheckbox = true;
  }

  final Key? key;
  final bool disabled;
  final IconData icon;
  final Function() onTap;
  final String text;
  final SeniorSquareButtonsMenuItemType type;

  late final bool isChecked;
  late final Function(bool?) onCheck;
  late final bool withCheckbox;
}

class SeniorSquareButtonsMenu extends StatelessWidget {
  const SeniorSquareButtonsMenu({
    required this.items,
    this.padding = const EdgeInsets.all(SeniorSpacing.xxsmall),
    Key? key,
  }) : super(key: key);

  final List<SeniorSquareButtonsMenuItemData> items;
  final EdgeInsetsGeometry padding;

  factory SeniorSquareButtonsMenu.grid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount,
    double crossAxisSpacing,
    double mainAxisSpacing,
  }) = _SeniorSquadButtonsMenuGrid;

  Gradient? _getBackgroundGradient(
    SeniorSquareButtonsMenuItemType type,
    bool disabled,
    SeniorTheme? theme,
    SeniorTheme defaultTheme,
  ) {
    if (type != SeniorSquareButtonsMenuItemType.primary || disabled) {
      return null;
    } else {
      final List<Color>? colors;

      switch (type) {
        case SeniorSquareButtonsMenuItemType.primary:
          colors = theme?.primarySquareButtonsMenuTheme?.style?.backgroundGradientColors ??
              defaultTheme.primarySquareButtonsMenuTheme?.style?.backgroundGradientColors ??
              SeniorColors.primaryGradientColors;
          break;
        case SeniorSquareButtonsMenuItemType.secondary:
          colors = theme?.secondarySquareButtonsMenuTheme?.style?.backgroundGradientColors ??
              defaultTheme.secondarySquareButtonsMenuTheme?.style?.backgroundGradientColors;
          break;
        case SeniorSquareButtonsMenuItemType.ghost:
          colors = theme?.ghostSquareButtonsMenuTheme?.style?.backgroundGradientColors ??
              defaultTheme.ghostSquareButtonsMenuTheme?.style?.backgroundGradientColors;
          break;
      }

      return colors != null
          ? LinearGradient(begin: Alignment.bottomLeft, end: Alignment.topRight, colors: colors)
          : null;
    }
  }

  Color _getBackgroundColor(
    SeniorSquareButtonsMenuItemType type,
    bool disabled,
    SeniorTheme? theme,
    SeniorTheme defaultTheme,
  ) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.primary:
        return disabled
            ? theme?.primarySquareButtonsMenuTheme?.style?.backgroundColor ??
                defaultTheme.primarySquareButtonsMenuTheme?.style?.backgroundColor ??
                SeniorColors.primaryColor
            : theme?.primarySquareButtonsMenuTheme?.style?.disabledBackgroundColor ??
                defaultTheme.primarySquareButtonsMenuTheme?.style?.disabledBackgroundColor ??
                SeniorColors.primaryColor;
      case SeniorSquareButtonsMenuItemType.secondary:
        return disabled
            ? theme?.secondarySquareButtonsMenuTheme?.style?.backgroundColor ??
                defaultTheme.secondarySquareButtonsMenuTheme?.style?.backgroundColor ??
                SeniorColors.secondaryColor600
            : theme?.secondarySquareButtonsMenuTheme?.style?.disabledBackgroundColor ??
                defaultTheme.secondarySquareButtonsMenuTheme?.style?.disabledBackgroundColor ??
                SeniorColors.secondaryColor600;
      case SeniorSquareButtonsMenuItemType.ghost:
        return disabled
            ? theme?.ghostSquareButtonsMenuTheme?.style?.backgroundColor ??
                defaultTheme.ghostSquareButtonsMenuTheme?.style?.backgroundColor ??
                Colors.transparent
            : theme?.ghostSquareButtonsMenuTheme?.style?.disabledBackgroundColor ??
                defaultTheme.ghostSquareButtonsMenuTheme?.style?.disabledBackgroundColor ??
                Colors.transparent;
    }
  }

  Color _getFontColor(
    SeniorSquareButtonsMenuItemType type,
    bool disabled,
    SeniorTheme? theme,
    SeniorTheme defaultTheme,
  ) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.primary:
        return disabled
            ? theme?.primarySquareButtonsMenuTheme?.style?.fontColor ??
                defaultTheme.primarySquareButtonsMenuTheme?.style?.fontColor ??
                SeniorColors.pureWhite
            : theme?.primarySquareButtonsMenuTheme?.style?.disabledFontColor ??
                defaultTheme.primarySquareButtonsMenuTheme?.style?.disabledFontColor ??
                SeniorColors.pureWhite;
      case SeniorSquareButtonsMenuItemType.secondary:
        return disabled
            ? theme?.secondarySquareButtonsMenuTheme?.style?.fontColor ??
                defaultTheme.secondarySquareButtonsMenuTheme?.style?.fontColor ??
                SeniorColors.pureWhite
            : theme?.secondarySquareButtonsMenuTheme?.style?.disabledFontColor ??
                defaultTheme.secondarySquareButtonsMenuTheme?.style?.disabledFontColor ??
                SeniorColors.pureWhite;
      case SeniorSquareButtonsMenuItemType.ghost:
        return disabled
            ? theme?.ghostSquareButtonsMenuTheme?.style?.fontColor ??
                defaultTheme.ghostSquareButtonsMenuTheme?.style?.fontColor ??
                SeniorColors.secondaryColor600
            : theme?.ghostSquareButtonsMenuTheme?.style?.disabledFontColor ??
                defaultTheme.ghostSquareButtonsMenuTheme?.style?.disabledFontColor ??
                SeniorColors.secondaryColor600;
    }
  }

  Widget _buildMenuItem(
    SeniorSquareButtonsMenuItemData item,
    double itemsSize,
    SeniorTheme? theme,
    SeniorTheme defaultTheme,
  ) {
    final double iconSize = itemsSize * 0.3;
    final bool smallSize = itemsSize < 100.0;

    return Container(
      height: itemsSize,
      width: itemsSize,
      child: Tooltip(
        message: item.text,
        padding: EdgeInsets.zero,
        child: Material(
          color: Colors.transparent,
          child: InkWell(
            borderRadius: BorderRadius.circular(
              SeniorRadius.xbig,
            ),
            onTap: item.onTap,
            child: DottedBorder(
              borderType: BorderType.RRect,
              radius: Radius.circular(
                SeniorRadius.xbig,
              ),
              padding: const EdgeInsets.all(0),
              color: item.type == SeniorSquareButtonsMenuItemType.ghost
                  ? _getFontColor(item.type, item.disabled, theme, defaultTheme)
                  : Colors.transparent,
              child: Ink(
                padding: const EdgeInsets.all(
                  SeniorSpacing.xsmall,
                ),
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(
                    SeniorRadius.xbig,
                  ),
                  gradient: _getBackgroundGradient(item.type, item.disabled, theme, defaultTheme),
                  color: _getBackgroundColor(item.type, item.disabled, theme, defaultTheme),
                ),
                child: Stack(
                  alignment: Alignment.center,
                  children: [
                    Visibility(
                      visible: item.withCheckbox && !smallSize,
                      child: Align(
                        alignment: Alignment.topLeft,
                        child: Container(
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(
                              SeniorRadius.small,
                            ),
                          ),
                          child: SeniorCheckbox(
                            value: item.isChecked,
                            onChanged: item.onCheck,
                          ),
                        ),
                      ),
                    ),
                    Container(
                      child: Column(
                        mainAxisAlignment: !smallSize ? MainAxisAlignment.center : MainAxisAlignment.center,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Icon(
                                item.icon,
                                color: _getFontColor(item.type, item.disabled, theme, defaultTheme),
                                size: iconSize,
                              ),
                            ],
                          ),
                          Visibility(
                            visible: !smallSize,
                            child: Padding(
                              padding: EdgeInsets.only(
                                top: itemsSize * 0.12,
                              ),
                              child: Text(
                                item.text,
                                textAlign: TextAlign.center,
                                overflow: TextOverflow.ellipsis,
                                maxLines: 2,
                                style: SeniorTypography.small(
                                  color: _getFontColor(item.type, item.disabled, theme, defaultTheme),
                                ),
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Container(
      height: 100,
      child: ListView.separated(
        padding: EdgeInsets.symmetric(
          horizontal: SeniorSpacing.normal,
        ),
        itemCount: items.length,
        scrollDirection: Axis.horizontal,
        separatorBuilder: (context, index) {
          return SizedBox(
            width: SeniorSpacing.xsmall,
          );
        },
        itemBuilder: (context, index) {
          final item = items[index];
          final double itemsSize = 100.0;
          return _buildMenuItem(item, itemsSize, theme, defaultTheme);
        },
      ),
    );
  }
}

class _SeniorSquadButtonsMenuGrid extends SeniorSquareButtonsMenu {
  late final int crossAxisCount;
  final double crossAxisSpacing;
  final double mainAxisSpacing;

  _SeniorSquadButtonsMenuGrid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount = 3,
    this.crossAxisSpacing = SeniorSpacing.xsmall,
    this.mainAxisSpacing = SeniorSpacing.xsmall,
  }) : super(key: key, items: items) {
    this.crossAxisCount = crossAxisCount <= 6 ? crossAxisCount : 6;
  }

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Container(
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          double menuWidth = constraints.constrainWidth();
          int slots = items.length;
          while (slots % crossAxisCount != 0) slots++;
          double itemsSize = (menuWidth - (8 * (crossAxisCount - 1))) / crossAxisCount;

          return GridView.count(
            shrinkWrap: true,
            crossAxisSpacing: crossAxisSpacing,
            mainAxisSpacing: mainAxisSpacing,
            crossAxisCount: crossAxisCount,
            physics: RangeMaintainingScrollPhysics(),
            children: items.map((item) => _buildMenuItem(item, itemsSize, theme, defaultTheme)).toList(),
          );
        },
      ),
    );
  }
}
