import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_text_field_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorTextField extends StatefulWidget {
  const SeniorTextField({
    Key? key,
    this.label,
    this.leftIcon,
    this.rightIcon,
    this.helper,
    this.hintText,
    this.disabled = false,
    this.initialValue,
    this.validator,
    this.maxLength,
    this.minLines,
    this.maxLines = 1,
    this.autocorrect = true,
    this.readOnly = false,
    this.onChanged,
    this.onTap,
    this.onEditingComplete,
    this.onFieldSubmitted,
    this.autofocus = false,
    this.controller,
    this.keyboardType,
    this.inputFormatters = const [],
    this.textInputAction,
    this.counterCaracterLabel,
    this.style,
  }) : super(key: key);

  final bool autocorrect;
  final bool autofocus;
  final TextEditingController? controller;
  final String? counterCaracterLabel;
  final bool disabled;
  final String? helper;
  final String? hintText;
  final String? initialValue;
  final List<TextInputFormatter> inputFormatters;
  final TextInputType? keyboardType;
  final String? label;
  final IconData? leftIcon;
  final int? maxLength;
  final int? maxLines;
  final int? minLines;
  final Function(String)? onChanged;
  final Function()? onEditingComplete;
  final Function(String?)? onFieldSubmitted;
  final Function()? onTap;
  final bool readOnly;
  final IconData? rightIcon;
  final SeniorTextFieldStyle? style;
  final TextInputAction? textInputAction;
  final FormFieldValidator<String>? validator;

  factory SeniorTextField.password({
    Key? key,
    bool autocorrect,
    bool autofocus,
    TextEditingController? controller,
    bool disabled,
    String? helper,
    String? hintText,
    String? initialValue,
    String? label,
    IconData? leftIcon,
    int? maxLength,
    Function(String)? onChanged,
    Function()? onEditingComplete,
    Function()? onTap,
    bool readOnly,
    SeniorTextFieldStyle? style,
    TextInputAction? textInputAction,
    FormFieldValidator<String>? validator,
  }) = _SeniorTextFieldPassword;

  @override
  _SeniorTextField createState() => _SeniorTextField();
}

class _SeniorTextField extends State<SeniorTextField> {
  late TextEditingController controller;
  late FocusNode focusNode;
  String? validMsg;
  bool hasFocus = false;
  bool hasError = false;
  String? errorMsg;

  @override
  void initState() {
    super.initState();
    focusNode = FocusNode();
    focusNode.addListener(onFocusChange);
    controller = widget.controller ?? TextEditingController();
    _internalValidate(value: widget.initialValue, validateAsError: false);
  }

  @override
  void dispose() {
    focusNode.dispose();
    super.dispose();
  }

  void onFocusChange() {
    setState(() {
      hasFocus = focusNode.hasFocus;
    });

    if (!hasFocus) {
      _internalValidate(value: controller.text);
    }
  }

  void _internalValidate({String? value, bool validateAsError = true}) {
    if (widget.validator != null) {
      validMsg = widget.validator!(value);

      if (validateAsError) {
        hasError = validMsg != null;
        errorMsg = validMsg;
      }
    }
    setState(() {});
  }

  String? _validate(String? value) {
    _internalValidate(value: value);
    return validMsg;
  }

  void onChanged(String value) {
    _internalValidate(value: value);

    if (widget.onChanged != null) {
      return widget.onChanged!(value);
    }
  }

  Widget? _buildFaIcon(IconData? icon, SeniorTheme? theme, SeniorTheme defaultTheme) {
    if (icon != null) {
      return Padding(
        padding: const EdgeInsets.symmetric(
          horizontal: SeniorSpacing.small,
          vertical: SeniorSpacing.normal,
        ),
        child: FaIcon(
          icon,
          size: SeniorTypographyIconSize.CTA,
          color: widget.style?.iconColor ??
              theme?.textFieldTheme?.style?.iconColor ??
              defaultTheme.textFieldTheme?.style?.iconColor ??
              SeniorColors.neutralColor900,
        ),
      );
    }
    return null;
  }

  Widget _buildTextField(SeniorTheme? theme, SeniorTheme defaultTheme) {
    var formatters = [
      ...widget.inputFormatters,
    ];

    if (widget.maxLength != null) {
      formatters.add(
        LengthLimitingTextInputFormatter(
          widget.maxLength,
        ),
      );
    }

    return AnimatedContainer(
      duration: const Duration(milliseconds: 200),
      // height: 56.0,
      decoration: BoxDecoration(
        color: hasFocus
            ? widget.style?.fillColor ??
                theme?.textFieldTheme?.style?.fillColor ??
                defaultTheme.textFieldTheme?.style?.fillColor ??
                SeniorColors.neutralColor100
            : Colors.transparent,
        borderRadius: BorderRadius.vertical(
          top: Radius.circular(SeniorRadius.xbig),
          bottom: Radius.circular(SeniorRadius.xsmall),
        ),
        border: Border.all(
          color: hasFocus
              ? hasError
                  ? widget.style?.errorColor ??
                      theme?.textFieldTheme?.style?.errorColor ??
                      defaultTheme.textFieldTheme?.style?.errorColor ??
                      SeniorColors.manchesterColorRed
                  : widget.style?.focusedBorderColor ??
                      theme?.textFieldTheme?.style?.focusedBorderColor ??
                      defaultTheme.textFieldTheme?.style?.focusedBorderColor ??
                      SeniorColors.primaryColor
              : Colors.transparent,
          width: 2.0,
        ),
      ),
      child: TextFormField(
        key: widget.key,
        textInputAction: widget.textInputAction,
        onFieldSubmitted: widget.onFieldSubmitted,
        style: SeniorTypography.text(
          color: widget.style?.textColor ??
              theme?.textFieldTheme?.style?.textColor ??
              defaultTheme.textFieldTheme?.style?.textColor ??
              SeniorColors.neutralColor900,
        ),
        onChanged: onChanged,
        onTap: widget.onTap,
        validator: _validate,
        focusNode: focusNode,
        minLines: widget.minLines,
        inputFormatters: formatters,
        maxLines: widget.maxLines,
        autocorrect: widget.autocorrect,
        readOnly: widget.readOnly,
        onEditingComplete: widget.onEditingComplete,
        autofocus: widget.autofocus,
        enabled: !widget.disabled,
        controller: controller,
        keyboardType: widget.keyboardType,
        decoration: InputDecoration(
          alignLabelWithHint: widget.maxLines != 1 && widget.maxLines != null,
          labelText: widget.label,
          labelStyle: hasFocus
              ? SeniorTypography.title(
                  color: hasError
                      ? widget.style?.errorColor ??
                          theme?.textFieldTheme?.style?.errorColor ??
                          defaultTheme.textFieldTheme?.style?.errorColor ??
                          SeniorColors.manchesterColorRed
                      : widget.style?.focusedLabelColor ??
                          theme?.textFieldTheme?.style?.focusedLabelColor ??
                          defaultTheme.textFieldTheme?.style?.focusedLabelColor ??
                          SeniorColors.primaryColor500,
                )
              : SeniorTypography.text(
                  color: hasError
                      ? widget.style?.errorColor ??
                          theme?.textFieldTheme?.style?.errorColor ??
                          defaultTheme.textFieldTheme?.style?.errorColor ??
                          SeniorColors.manchesterColorRed
                      : widget.style?.unfocusedLabelColor ??
                          theme?.textFieldTheme?.style?.unfocusedLabelColor ??
                          defaultTheme.textFieldTheme?.style?.unfocusedLabelColor ??
                          SeniorColors.neutralColor900,
                ),
          prefixIcon: _buildFaIcon(widget.leftIcon, theme, defaultTheme),
          suffixIcon: _buildFaIcon(widget.rightIcon, theme, defaultTheme),
          filled: true,
          fillColor: widget.style?.fillColor ??
              theme?.textFieldTheme?.style?.fillColor ??
              defaultTheme.textFieldTheme?.style?.fillColor ??
              SeniorColors.neutralColor100,
          focusedBorder: InputBorder.none,
          errorStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          focusedErrorBorder: InputBorder.none,
          hintText: widget.hintText,
          counterStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          hintStyle: SeniorTypography.text(
            color: widget.style?.hintColor ??
                theme?.textFieldTheme?.style?.hintColor ??
                defaultTheme.textFieldTheme?.style?.hintColor ??
                SeniorColors.neutralColor600,
          ),
        ),
      ),
    );
  }

  Widget _buildHelper(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return Padding(
      padding: const EdgeInsets.only(
        top: SeniorSpacing.xxsmall,
        left: SeniorSpacing.normal,
      ),
      child: Text(
        errorMsg ?? widget.helper ?? '',
        overflow: TextOverflow.ellipsis,
        style: SeniorTypography.helper(
          color: hasError
              ? widget.style?.errorColor ??
                  theme?.textFieldTheme?.style?.errorColor ??
                  defaultTheme.textFieldTheme?.style?.errorColor ??
                  SeniorColors.manchesterColorRed
              : widget.style?.helperColor ??
                  theme?.textFieldTheme?.style?.helperColor ??
                  defaultTheme.textFieldTheme?.style?.helperColor ??
                  SeniorColors.neutralColor700,
        ),
      ),
    );
  }

  Widget _buildCounter(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return widget.maxLength == null
        ? SizedBox.shrink()
        : Padding(
            padding: const EdgeInsets.symmetric(
              horizontal: SeniorSpacing.normal,
              vertical: SeniorSpacing.xxsmall,
            ),
            child: Text(
              '${widget.maxLength! - controller.text.length} ${widget.counterCaracterLabel ?? ''}',
              style: SeniorTypography.helper(
                color: widget.style?.counterColor ??
                    theme?.textFieldTheme?.style?.counterColor ??
                    defaultTheme.textFieldTheme?.style?.counterColor ??
                    SeniorColors.neutralColor700,
              ),
            ),
          );
  }

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        _buildTextField(theme, defaultTheme),
        Row(
          children: [
            Expanded(child: _buildHelper(theme, defaultTheme)),
            _buildCounter(theme, defaultTheme),
          ],
        ),
      ],
    );
  }
}

class _SeniorTextFieldPassword extends SeniorTextField {
  _SeniorTextFieldPassword({
    Key? key,
    bool autocorrect = false,
    bool autofocus = false,
    TextEditingController? controller,
    bool disabled = false,
    String? helper,
    String? hintText,
    String? initialValue,
    String? label,
    IconData? leftIcon,
    int? maxLength,
    Function(String)? onChanged,
    Function()? onEditingComplete,
    Function(String?)? onFieldSubmitted,
    Function()? onTap,
    bool readOnly = false,
    SeniorTextFieldStyle? style,
    TextInputAction? textInputAction,
    FormFieldValidator<String>? validator,
  }) : super(
          key: key,
          autocorrect: autocorrect,
          autofocus: autofocus,
          controller: controller,
          disabled: disabled,
          helper: helper,
          hintText: hintText,
          initialValue: initialValue,
          keyboardType: TextInputType.visiblePassword,
          label: label,
          leftIcon: leftIcon,
          maxLength: maxLength,
          onChanged: onChanged,
          onEditingComplete: onEditingComplete,
          onFieldSubmitted: onFieldSubmitted,
          onTap: onTap,
          readOnly: readOnly,
          style: style,
          textInputAction: textInputAction,
          validator: validator,
        );

  @override
  _SeniorTextField createState() => _SeniorTextFieldPasswordState();
}

class _SeniorTextFieldPasswordState extends _SeniorTextField {
  bool obscureText = true;

  @override
  Widget _buildTextField(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return AnimatedContainer(
      duration: const Duration(milliseconds: 200),
      height: 56.0,
      decoration: BoxDecoration(
        color: hasFocus
            ? widget.style?.focusedBorderColor ??
                theme?.textFieldTheme?.style?.focusedBorderColor ??
                defaultTheme.textFieldTheme?.style?.focusedBorderColor ??
                SeniorColors.neutralColor100
            : Colors.transparent,
        borderRadius: BorderRadius.vertical(
          top: Radius.circular(SeniorRadius.xbig),
          bottom: Radius.circular(SeniorRadius.xsmall),
        ),
        border: Border.all(
          color: hasFocus
              ? hasError
                  ? widget.style?.errorColor ??
                      theme?.textFieldTheme?.style?.errorColor ??
                      defaultTheme.textFieldTheme?.style?.errorColor ??
                      SeniorColors.manchesterColorRed
                  : widget.style?.focusedBorderColor ??
                      theme?.textFieldTheme?.style?.focusedBorderColor ??
                      defaultTheme.textFieldTheme?.style?.focusedBorderColor ??
                      SeniorColors.primaryColor
              : Colors.transparent,
          width: 2.0,
        ),
      ),
      child: TextFormField(
        key: widget.key,
        style: SeniorTypography.text(
          color: widget.style?.textColor ??
              theme?.textFieldTheme?.style?.textColor ??
              defaultTheme.textFieldTheme?.style?.textColor ??
              SeniorColors.neutralColor900,
        ),
        onChanged: onChanged,
        onTap: widget.onTap,
        validator: _validate,
        focusNode: focusNode,
        minLines: widget.minLines,
        maxLines: widget.maxLines,
        autocorrect: widget.autocorrect,
        readOnly: widget.readOnly,
        onEditingComplete: widget.onEditingComplete,
        onFieldSubmitted: widget.onFieldSubmitted,
        autofocus: widget.autofocus,
        enabled: !widget.disabled,
        controller: controller,
        keyboardType: widget.keyboardType,
        textInputAction: widget.textInputAction ?? TextInputAction.done,
        obscureText: obscureText,
        decoration: InputDecoration(
          labelText: widget.label,
          labelStyle: hasFocus
              ? SeniorTypography.title(
                  color: hasError
                      ? widget.style?.errorColor ??
                          theme?.textFieldTheme?.style?.errorColor ??
                          defaultTheme.textFieldTheme?.style?.errorColor ??
                          SeniorColors.manchesterColorRed
                      : widget.style?.focusedLabelColor ??
                          theme?.textFieldTheme?.style?.focusedLabelColor ??
                          defaultTheme.textFieldTheme?.style?.focusedLabelColor ??
                          SeniorColors.primaryColor500,
                )
              : SeniorTypography.text(
                  color: hasError
                      ? widget.style?.errorColor ??
                          theme?.textFieldTheme?.style?.errorColor ??
                          defaultTheme.textFieldTheme?.style?.errorColor ??
                          SeniorColors.manchesterColorRed
                      : widget.style?.unfocusedLabelColor ??
                          theme?.textFieldTheme?.style?.unfocusedLabelColor ??
                          defaultTheme.textFieldTheme?.style?.unfocusedLabelColor ??
                          SeniorColors.neutralColor900,
                ),
          prefixIcon: _buildFaIcon(widget.leftIcon, theme, defaultTheme),
          suffixIcon: buildPasswordEyeIcon(theme, defaultTheme),
          filled: true,
          fillColor: widget.style?.fillColor ??
              theme?.textFieldTheme?.style?.fillColor ??
              defaultTheme.textFieldTheme?.style?.fillColor ??
              SeniorColors.neutralColor100,
          focusedBorder: InputBorder.none,
          errorBorder: InputBorder.none,
          errorStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          counterStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          focusedErrorBorder: InputBorder.none,
          hintText: widget.hintText,
          hintStyle: SeniorTypography.text(
            color: widget.style?.hintColor ??
                theme?.textFieldTheme?.style?.hintColor ??
                defaultTheme.textFieldTheme?.style?.hintColor ??
                SeniorColors.neutralColor600,
          ),
        ),
      ),
    );
  }

  Widget buildPasswordEyeIcon(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return Padding(
      padding: const EdgeInsets.symmetric(
        horizontal: SeniorSpacing.xxsmall,
      ),
      child: GestureDetector(
        onTap: () {
          setState(() {
            obscureText = !obscureText;
          });
        },
        child: Align(
          alignment: Alignment.centerRight,
          child: Container(
            decoration: BoxDecoration(
              shape: BoxShape.circle,
            ),
            padding: const EdgeInsets.all(SeniorSpacing.small),
            child: FaIcon(
              obscureText ? FontAwesomeIcons.eye : FontAwesomeIcons.eyeSlash,
              size: SeniorTypographyIconSize.CTA,
              color: widget.style?.iconColor ??
                  theme?.textFieldTheme?.style?.iconColor ??
                  defaultTheme.textFieldTheme?.style?.iconColor ??
                  SeniorColors.neutralColor900,
            ),
          ),
        ),
      ),
    );
  }
}
