"""Validate the Custom Issues of a project"""

import sys
from datetime import datetime, timedelta
from typing import List

from colorama import Fore


sys.path.append("./senior-ci")

from common import (
    end_collapsible_section,
    exit_message,
    get_env_variable,
    print_message,
    start_collapsible_section,
)
from common.graphql_client import GraphqlClient
from common.validations.check_deprecated_primitives import CheckDeprecatedPrimitives
from common.validations.check_tags import CheckTags


class CustomIssue:
    """Entity of a Custom Issue"""

    rule: str
    must_be_fixed_until: datetime
    description: str
    notification_text: str

    def __init__(self, client_graphql: GraphqlClient = GraphqlClient()) -> None:
        self.client_graphql = client_graphql

    def create(self):
        """Create a CustomIssue for project"""

        query = """
            mutation ($projectId: Int!, $customIssue: CustomIssueInput!) {
                createCustomCiIssue(
                    customIssueInfo: $customIssue
                    gitlabInfo:{
                        projectId: $projectId
                    }
                ) {
                    success
                    message
                }
            }
        """

        params = {
            "projectId": int(get_env_variable("CI_PROJECT_ID")),
            "customIssue": self.convert_to_json(),
        }

        self.client_graphql.call(query, params)

    def convert_from_json(self, payload):
        """Convert a payload to this class"""

        self.rule = payload["rule"]
        self.must_be_fixed_until = payload["mustBeFixedUntil"]

        if self.must_be_fixed_until:
            self.must_be_fixed_until = datetime.strptime(
                self.must_be_fixed_until, "%Y-%m-%dT%H:%M:%S"
            )

        self.description = payload["description"]
        self.notification_text = payload["notificationText"]

        return self

    def convert_to_json(self):
        """Convert this class to a json"""

        return {
            "rule": self.rule,
            "mustBeFixedUntil": self.must_be_fixed_until.strftime("%Y-%m-%dT%H:%M:%S"),
            "description": self.description,
            "notificationText": self.notification_text,
        }


class ProjectIssuesStatus:
    """Entity of a Project Issues Status"""

    should_break_job: bool
    issues: List[CustomIssue]

    def __init__(self, client_graphql: GraphqlClient = GraphqlClient()) -> None:
        self.client_graphql = client_graphql
        self._get_project_issue_status()

    def show_status(self):
        """Print the status of a job and/or project"""

        print_message("Validando existência de alguma pendência no projeto.")
        print(" ")

        if self.issues:
            section_id = start_collapsible_section(
                "Existem algumas pendências a serem resolvidas, segue relação:",
            )

            for issue in self._get_required_issues():
                must_be_fixed_until = issue.must_be_fixed_until
                description = issue.description

                date_to_fix_message = None

                if must_be_fixed_until:
                    date_to_fix = issue.must_be_fixed_until.strftime("%d/%m/%Y")

                    date_to_fix_message = (
                        f"Data de corte para quebrar o job: {date_to_fix}"
                    )

                print_message(
                    f"{description} {date_to_fix_message if date_to_fix_message else ''}",
                    Fore.RED if self.should_break_job else Fore.LIGHTYELLOW_EX,
                )

            if self.should_break_job:
                exit_message(
                    "O job foi impedido de continuar devido as pendências acima. :x"
                )

            print(" ")

            print_message(
                "Nenhuma das pendências acima quebrará este job, porém é importante ajustá-las antes que se tornem impeditivas ;)",
                Fore.YELLOW,
            )

            end_collapsible_section(section_id)
        else:
            print_message("Nenhuma pendência encontrada, parabéns!", Fore.GREEN)

        print(" ")

    def _get_required_issues(self):
        self.issues.sort(
            key=lambda issue: issue.must_be_fixed_until
            if issue.must_be_fixed_until
            else datetime.now()
        )

        required_issues = self.issues

        if self.should_break_job:
            required_issues = list(
                filter(lambda issue: issue.must_be_fixed_until is None, self.issues)
            )

            if not required_issues:
                required_issues = list(
                    filter(
                        lambda issue: issue.must_be_fixed_until < datetime.now(),
                        self.issues,
                    )
                )

        return required_issues

    def _get_project_issue_status(self):
        query = """
            query ($projectId: Int!, $jobStage: String, $commitSha: String) {
                projectIssuesStatus(
                    projectId: $projectId,
                    jobStage: $jobStage,
                    commitSha: $commitSha
                ) {
                    shouldBreakJob
                    issues {
                        rule
                        mustBeFixedUntil
                        description
                        notificationText
                    }
                }
            }
        """

        params = {
            "projectId": int(get_env_variable("CI_PROJECT_ID")),
            "jobStage": get_env_variable("CI_JOB_STAGE"),
            "commitSha": get_env_variable("CI_COMMIT_SHA"),
        }

        result = self.client_graphql.call(query, params)

        project_issues_status_result = result["projectIssuesStatus"]

        self.convert_from_json(project_issues_status_result)

    def convert_from_json(self, payload):
        """Convert a payload to this class"""

        self.should_break_job = payload["shouldBreakJob"]
        self.issues = list(
            map(
                lambda issueResult: CustomIssue().convert_from_json(issueResult),
                payload["issues"],
            )
        )

        return self


skip_check_deprecated_primitives = get_env_variable("SKIP_CHECK_DEPRECATED_PRIMITIVES")

graphql_client = GraphqlClient()

if not skip_check_deprecated_primitives:
    check_tags = CheckDeprecatedPrimitives(graphql_client)
    result_check = check_tags.check()

    if not result_check.success:
        custom_issues = CustomIssue(graphql_client)
        custom_issues.rule = "HAS_DEPRECATED_PRIMITIVES"
        custom_issues.description = "Foi encontrado primitivas deprecadas no projeto."
        custom_issues.notification_text = result_check.reason

        today = datetime.today()
        next_two_week = today + timedelta(days=15)

        must_be_fixed_until = result_check.last_deprecated_date

        if must_be_fixed_until < today or must_be_fixed_until < next_two_week:
            must_be_fixed_until = today + timedelta(days=60)

        custom_issues.must_be_fixed_until = must_be_fixed_until

        custom_issues.create()

skip_check_tags = get_env_variable("SKIP_CHECK_TAGS")

if not skip_check_tags:
    check_tags = CheckTags(graphql_client)
    result_check = check_tags.check()

    if not result_check.success:
        custom_issues = CustomIssue(graphql_client)
        custom_issues.rule = "MISSING_SONAR_TAGS"
        custom_issues.description = "Não foram encontradas tags do sonar no projeto."
        custom_issues.notification_text = result_check.reason

        today = datetime.today()
        next_week = today + timedelta(days=7)

        custom_issues.must_be_fixed_until = next_week

        custom_issues.create()

skip_issue_validate = get_env_variable("SKIP_ISSUE_VALIDATE")

if not skip_issue_validate:
    project_issues_status = ProjectIssuesStatus(graphql_client)
    project_issues_status.show_status()
