#!/bin/bash
set -e # Exit at first error


#Init Variables
HAS_CACHE="false"
CACHE_S3_ADDR="s3://senior-ci-cache-build-backend/ci-cache"
SONAR_URL="http://sonar.senior.com.br"


# Import release helpers
. senior-ci/common/release-helpers.sh

# Import notification helpers
. senior-ci/common/notification-helpers.sh

# Set $PARALLEL
. senior-ci/common/parallel-helper.sh

# Senior CI Extensions
. senior-ci/common/senior-ci-extensions-helpers.sh

# Import npm S3 helpers
. senior-ci/common/s3cache-helpers.sh

# Import docker helpers
. senior-ci/common/docker-helpers.sh

# Print de variáveis para diagnóstico
print_job_vars() {
  echo ""
  echo "----------------------------------------------------------------"
  echo "[CI] Variáveis:"
  echo "[CI] - SCI_VERSION_TAG_BACKEND=$SCI_VERSION_TAG_BACKEND"
  echo "[CI] - NO_PARALLEL=$NO_PARALLEL"
  echo "[CI] - VERSION=$VERSION"
  echo "[CI] - SONAR_URL=$SONAR_URL"
  echo "[CI] - TEAMS_HOOK=$TEAMS_HOOK"
  echo "[CI] - CI_COMMIT_TAG=$CI_COMMIT_TAG"
  echo "----------------------------------------------------------------"
  echo ""
}

use_snapshot_timestamp_version() {
  TS=$(date +"%s")
  VERSION="$CI_COMMIT_REF_SLUG-$TS-SNAPSHOT"
}

use_snapshot_version() {
  if grep --fixed-strings --quiet "<tycho-version>" pom.xml; then
    VERSION="$(grep -m 1 "<version>" pom.xml | cut -d ">" -f 2 | awk -vRS='<|>' NR%2).${CI_COMMIT_REF_SLUG//./}-v$(date +%Y%m%d%H%M)"
  else
    VERSION="$CI_COMMIT_REF_SLUG-SNAPSHOT"
  fi
}

set_version() {
  if grep --fixed-strings --quiet "<tycho-version>" pom.xml; then
    [[ $VERSION =~ ^v.* ]] && VERSION=$(echo $CI_COMMIT_REF_SLUG |awk '{gsub("v","");gsub(/\-/,".");}1')
    echo "[CI] Alterando versão com tycho-version:1.7.0. Nova versão - $VERSION"
    mvn org.eclipse.tycho:tycho-versions-plugin:1.7.0:set-version -DnewVersion=$VERSION
  else
    echo "[CI] Alterando versão com mvn versions:set. Nova versão - $VERSION"
    mvn versions:set --batch-mode --errors -DgenerateBackupPoms=false -DnewVersion=$VERSION
  fi
}

normalize_ci_commit_tag() {
  CI_COMMIT_TAG="${CI_COMMIT_TAG//-/.}"
  CI_COMMIT_TAG="${CI_COMMIT_TAG:1}"
}

publish() {
  docker_login

  if [[ "$VERSION" == *-SNAPSHOT ]] && [[ "$HAS_CACHE" == "true" ]] ; then
    echo "[CI] Realizando o envio dos artefatos ao servidor usando cache de build"
    #Parametro Dmaven.compiler.useIncrementalCompilation setado como false para que o mesmo use o cache copiado
    # Feito isso em decorrencia de um bug do maven (https://issues.apache.org/jira/browse/MCOMPILER-209)
    mvn $PARALLEL deploy --batch-mode --errors -DretryFailedDeploymentCount=5 -Dmaven.compiler.useIncrementalCompilation=false -DskipTests=true
  else
    echo "[CI] Realizando a compilação dos artefatos"
    mvn $PARALLEL clean package --batch-mode --errors --update-snapshots -DskipTests=true

    echo "[CI] Realizando o envio dos artefatos ao servidor"
    mvn $PARALLEL deploy --batch-mode --errors -DretryFailedDeploymentCount=5 -DskipTests=true
  fi

}

publish_snapshot(){
    print_job_vars
    set_version
    create_dockerhub_repo
    publish
}


# Execução dos jobs
getopts "bstrpx" job
case "$job" in
  b) echo "[CI] Executando build, testes e análise do Sonar..."
    python3 senior-ci/common/validations/validate_issues.py

    docker_login
    call_extension "before_build"

    use_snapshot_version
    print_job_vars
    set_version
     
    if [ "$SKIP_VERIFY_ON_BUILD" = "true" ];
    then
      echo "[CI] Executando build sem verify - mvn $PARALLEL clean test --batch-mode --errors --update-snapshots"
      mvn $PARALLEL clean test --batch-mode --errors --update-snapshots
    else
      echo "[CI] Executando build com verify - mvn $PARALLEL clean test verify --batch-mode --errors --update-snapshots"
      mvn $PARALLEL clean test verify --batch-mode --errors --update-snapshots
    fi    
    
    if [ -n "$CACHE_FILE_PATTERN" ]; then
      createS3BuildCache $CACHE_FILE_PATTERN
    fi

    call_extension "after_build"


    call_extension "before_sonar_preview"
    mvn $PARALLEL sonar:sonar --batch-mode --errors --update-snapshots -Dsonar.host.url=$SONAR_URL -Dsonar.issuesReport.console.enable=true -Dsonar.projectName=$CI_PROJECT_NAME -Dsonar.gitlab.project_id=$CI_PROJECT_ID -Dsonar.analysis.mode=preview -Dsonar.gitlab.commit_sha=$CI_BUILD_REF -Dsonar.gitlab.ref_name=$CI_BUILD_REF_NAME
    call_extension "after_sonar_preview"
    ;;

  s) echo "[CI] Realizando release: SNAPSHOT"
    call_extension "before_release_snapshot"

    use_snapshot_version
    getS3BuildCache
    publish_snapshot

    call_extension "after_release_snapshot"
    ;;

  t) echo "[CI] Realizando release: SNAPSHOT TIMESTAMP"
    call_extension "before_release_snapshot"

    use_snapshot_timestamp_version
    publish_snapshot

    call_extension "after_release_snapshot"
    ;;

  r) echo "[CI] Realizando release: $2"
    python3 senior-ci/common/validations/validate_issues.py

    unshallow_repo

    get_next_version $2

    create_release_branch

    call_extension "before_release"

    call_extension "before_change_changelog"

    update_changelog

    call_extension "after_change_changelog"

    set_version

    create_release_tag

    call_extension "after_release"

    echo "[CI] Criação da release versão $VERSION realizado com sucesso. O job 'package' será executado automaticamente para publicar esta versão no Maven."

    notify_release_teams_channel
    ;;
  p) echo "[CI] Publicação da versão..."
    call_extension "before_packaging"

    normalize_ci_commit_tag
    print_job_vars
    publish

    USER_DEPLOY_MAIL="${GITLAB_USER_EMAIL%@senior.com.br}"
    DEPLOY_TEAMS_MESSAGE="O usuário $USER_DEPLOY_MAIL liberou a versão <$CI_PROJECT_URL/blob/$CI_COMMIT_REF_NAME/CHANGELOG.md#anchor-$VERSION_WITHOUT_DOTS|$VERSION> do projeto $CI_PROJECT_NAME."

    call_extension "after_packaging"

    notify_text_teams_channel "$DEPLOY_TEAMS_MESSAGE"

    echo "[CI] Versão $VERSION publicada no Maven com sucesso!"
    ;;

  x) echo "[CI] Realizando análise e publicação dos resultados no SonarQube - $SONAR_URL"
    VERSION="$CI_COMMIT_REF_SLUG"

    call_extension "before_sonar_send"

    set_version


    mvn $PARALLEL clean compile test sonar:sonar --batch-mode --errors --update-snapshots -Dmaven.test.failure.ignore=true -Dsonar.links.scm=$CI_PROJECT_URL -Dsonar.host.url=$SONAR_URL -Dsonar.issuesReport.console.enable=true -Dsonar.gitlab.project_id=$CI_PROJECT_ID
    mvn $PARALLEL sonar:sonar --batch-mode --errors -Dsonar.links.scm=$CI_PROJECT_URL -Dsonar.host.url=$SONAR_URL -Dsonar.issuesReport.console.enable=true -Dsonar.gitlab.project_id=$CI_PROJECT_ID

    call_extension "after_sonar_send"
    ;;

esac
