import getopt
import json
from pathlib import Path
from configparser import ConfigParser
from colorama import Fore

from common import print_message, get_env_variable
from common.getopt_helper import GetOptBuilder, Param

from jira_manager import JiraManager
from changelog_reader import ChangelogReader


class CreateSreIssue(GetOptBuilder):
    def __init__(self, argv):
        builder = super()
        builder.__init__(argv)

        tag_param = Param("tag", default_value=get_env_variable("CI_COMMIT_REF_SLUG"))
        repository_param = Param(
            "repository", default_value=get_env_variable("CI_PROJECT_NAME")
        )
        module_param = Param("module", default_value="")
        changelog_base_path_param = Param("changelog_base_path", "c", default_value=".")
        sre_team_param = Param("sre_team", "s", is_required=True)

        builder.add_params(
            [
                tag_param,
                repository_param,
                module_param,
                changelog_base_path_param,
                sre_team_param,
            ]
        )

        builder.build()

        self.jira_manager = JiraManager()

        self.tag = tag_param.get_param_value()
        self.repository = repository_param.get_param_value()
        self.module = module_param.get_param_value()
        self.changelog_base_path = changelog_base_path_param.get_param_value()
        self.sre_team = sre_team_param.get_param_value()

        self.changelog_reader = ChangelogReader(
            self.changelog_base_path, self.tag, self.jira_manager
        )

    def exec(self):
        """Reads the issues from a given version in changelog and creates an 'deploy issue'"""

        changelog_issues = self.changelog_reader.read_issues()

        if not changelog_issues:
            print_message(
                f"Nenhuma task encontrada no CHANGELOG para a tag {self.tag}",
                Fore.LIGHTYELLOW_EX,
            )
            return

        sre_issue_type = (
            "Bug"
            if any(issue.fields.issuetype.name == "Bug" for issue in changelog_issues)
            else "Story"
        )

        deploy_name_custom_field = (
            "customfield_12002"
            if "frontend" in self.repository
            else "customfield_12001"
        )

        sre_issue = self._create_sre_issue(sre_issue_type, deploy_name_custom_field)

        print_message(
            f"Task criada para o SRE: {sre_issue.key} - {sre_issue.fields.summary} ({self.tag})",
            Fore.GREEN,
        )

        for issue in changelog_issues:
            if self.jira_manager.create_link(
                "Dependência entre atividades", issue, sre_issue
            ):
                print_message(f"Link criado: {issue.key} - {sre_issue.key}", Fore.GREEN)

    def _create_sre_issue(self, sre_issue_type, deploy_name_custom_field):
        package_json_path = Path("package.json")
        sdl_properties_path = Path("sdl.properties")

        if "frontend" in self.repository and package_json_path.is_file():
            with package_json_path.open("r", encoding="utf-8") as package_data:
                json_data = json.load(package_data)

            app = json_data["project"]["app"]
            domain = json_data["project"]["domain"]

            service_path = f"{app}/{domain}/{self.repository}:{self.tag}"
        elif sdl_properties_path.is_file():
            with sdl_properties_path.open("r", encoding="utf-8") as properties_data:
                config_string = "[DEFAULT]\n" + properties_data.read()

            config = ConfigParser()
            config.read_string(config_string)

            app = config["DEFAULT"]["generator.app.name"]

            service_path = f"{app}:{self.tag}"
        else:
            service_path = f"{self.repository}:{self.tag}"

        jira_issue = {
            "project": {"key": self.sre_team},
            "summary": f"Deploy {self.repository}",
            "issuetype": {"name": "Deploy"},
            "labels": [sre_issue_type, self.module],
            "description": self.changelog_reader.changelog_url,
            deploy_name_custom_field: [service_path],
            "customfield_11100": [{"value": "Produção - platform.senior.com.br"}],
        }

        return self.jira_manager.create_issue(jira_issue)
