#!/bin/bash
set -e # Exit at first error

#Init Variables
HAS_CACHE="false"
CACHE_S3_ADDR="s3://senior-ci-cache-build-backend/ci-cache"
HELP_MESSAGE="Caso tenha qualquer dúvida sobre o processo, consulte nossa Wiki - https://git.senior.com.br/engenharia/senior-ci/wikis/home"


# Import release helpers
. senior-ci/common/release-helpers.sh

# Import notification helpers
. senior-ci/common/notification-helpers.sh

# Set $PARALLEL
. senior-ci/common/parallel-helper.sh

# Senior CI Extensions
. senior-ci/common/senior-ci-extensions-helpers.sh

# Import npm S3 helpers
. senior-ci/common/s3cache-helpers.sh

# Import docker helpers
. senior-ci/common/docker-helpers.sh

# Propriedades da SDL
determine_properties() {
    OUTPUT=$(grep "^generator.java.output\+[[:space:]]*\+=" sdl.properties | cut -d "=" -f 2 | tr -d '[[:space:]]')
    : ${OUTPUT:=java}
    CIFRIENDLY=$(grep "^generator.java.ciFriendlyVersion\+[[:space:]]*\+=" sdl.properties | head -1 | cut -d "=" -f 2 | tr -d '[[:space:]]')
    : ${CIFRIENDLY:=false}
}

# Versionamento
CHANGELIST=""

use_snapshot_version() {
    VERSION="$CI_COMMIT_REF_SLUG-SNAPSHOT"
    CHANGELIST="-SNAPSHOT"
}

use_snapshot_timestamp_version() {
    TS=$(date +"%s")
    VERSION="$CI_COMMIT_REF_SLUG-$TS-SNAPSHOT"
    CHANGELIST="-SNAPSHOT"
}

set_version() {
    if [ "$CIFRIENDLY" != "true" ]; then
      sed -i "s/generator.app.version=.*/generator.app.version=${VERSION}/" sdl.properties
    fi

    mvn versions:set --batch-mode --errors -DgenerateBackupPoms=false -DnewVersion=$VERSION
    mvn versions:set --batch-mode --errors -DgenerateBackupPoms=false -DnewVersion=$VERSION --file $OUTPUT
}

versioned_mvn() {
    CMD="mvn"
    if [ "$CIFRIENDLY" == "true" ]; then
      if [ "$CHANGELIST" == "" ]; then
        if [ "$VERSION" == "" ]; then
          VERSION=$(git tag --sort=-v:refname | grep "^v[0-9]\{1,\}-[0-9]\{1,\}-[0-9]\{1,\}$" | head -1 | sed "s/-\([[:digit:]]\)/\.\1/g")
          VERSION="${VERSION:1}"
          echo "[CI] - VERSION=$VERSION"
        fi
	    CMD="$CMD -Drevision=$VERSION -Dsha1= -Dchangelist="
      else
	    CMD="$CMD -Drevision=$CI_COMMIT_REF_SLUG -Dsha1= -Dchangelist=$CHANGELIST"
      fi
    fi
    CMD="$CMD $@"
	echo "$CMD"
    $CMD
}

# SonarQube
SONAR_URL="http://sonar.senior.com.br"
SONAR_PROJECTS="!client,!server"

if [ -n "$SCI_ENABLE_SONAR_SDL" ]; then
  SONAR_PROJECTS="!client"
fi

MESSAGE_HELP="Caso tenha qualquer dúvida sobre o processo, consulte nossa Wiki - https://git.senior.com.br/engenharia/senior-ci/wikis/home"

# Print de variáveis para diagnóstico
print_job_vars() {
    echo ""
    echo "----------------------------------------------------------------"
    echo "[CI] Variáveis:"
    echo "[CI] - SCI_VERSION_TAG_BACKEND=$SCI_VERSION_TAG_BACKEND"
    echo "[CI] - SCI_ENABLE_SONAR_SDL=$SCI_ENABLE_SONAR_SDL"
    echo "[CI] - NO_PARALLEL=$NO_PARALLEL"
    echo "[CI] - OUTPUT=$OUTPUT"
    echo "[CI] - CIFRIENDLY=$CIFRIENDLY"
    echo "[CI] - CHANGELIST=$CHANGELIST"
    echo "[CI] - VERSION=$VERSION"
    echo "[CI] - SONAR_URL=$SONAR_URL"
    echo "[CI] - SONAR_PROJECTS=$SONAR_PROJECTS"
    echo "[CI] - TEAMS_HOOK=$TEAMS_HOOK"
    echo "----------------------------------------------------------------"
    echo ""
}


# Chamadas mvn
generate_sources() {
    echo "[CI] Executando generate-sources..."
    versioned_mvn $PARALLEL clean generate-sources --batch-mode --errors --update-snapshots
}

publish() {
    # Utilizado no package e no releaseSnapshot

    getS3BuildCache

    docker_login

    echo "[CI] Realizando a compilação dos artefatos"

    if [[ "$VERSION" == *-SNAPSHOT ]] && [[ "$HAS_CACHE" == "true" ]] ; then
        echo "[CI] Usando cache existente."
        versioned_mvn $PARALLEL dockerfile:build -DskipTests=true -Dmaven.compiler.useIncrementalCompilation=true --file $OUTPUT/impl

    else
        echo "[CI] Buildando o projeto novamente."
        versioned_mvn $PARALLEL install --batch-mode --errors -DskipTests=true --file $OUTPUT
    fi

    if [ "$1" == "docker" ]; then
      if [ -z "$SKIP_IMAGE_SCAN" ]; then
        if [ -z "$SVC_IMAGE" ]; then
          IMAGE_NAME=`cat sdl.properties | sed -n -e 's/generator.app.name=//p'`
          IMAGE_NAME=`echo ${IMAGE_NAME%%[[:cntrl:]]} | xargs`
        else
          IMAGE_NAME=$SVC_IMAGE
        fi

        TAG="${CI_COMMIT_TAG:-$CI_COMMIT_REF_SLUG}"
        VERSION_FORMATED=$(echo $TAG |awk '{gsub("v","");gsub(/\-/,".");}1')
        IMAGEM=seniorsistemas/${IMAGE_NAME/$'\r'/}:${VERSION_FORMATED}
        if [[ "$(docker images -q $IMAGEM 2> /dev/null)" == "" ]]; then
          echo "[CI] Imagem $IMAGEM não encontrada, alterando para repositorio alternativo. "
          IMAGEM=seniorsa/${IMAGE_NAME/$'\r'/}:${VERSION_FORMATED}
        fi

        echo "[CI] Realizando a analise de segurança da imagem: $IMAGEM"
        trivy --severity HIGH,CRITICAL $IMAGEM
      fi
    fi

    create_dockerhub_repo

    echo "[CI] Realizando o envio dos artefatos ao servidor"
    if [[ "$VERSION" == *-SNAPSHOT ]] && [[ "$HAS_CACHE" == "true" ]] ; then
      # Parametro Dmaven.compiler.useIncrementalCompilation setado como false para que o mesmo use o cache copiado
      # Feito isso em decorrencia de um bug do maven (https://issues.apache.org/jira/browse/MCOMPILER-209)
      versioned_mvn $PARALLEL deploy --batch-mode --errors -DretryFailedDeploymentCount=5 -Dmaven.compiler.useIncrementalCompilation=false -DskipTests=true --file $OUTPUT
    else
      versioned_mvn $PARALLEL deploy --batch-mode --errors -DretryFailedDeploymentCount=5 -DskipTests=true --file $OUTPUT
    fi
}

publish_snapshot(){
    call_extension "before_release_snapshot"
    determine_properties
    print_job_vars
    generate_sources
    set_version
    publish
    call_extension "after_release_snapshot"
}

generateS3BuildCache() {
  CACHE_FILE_NAME="BUILD_CACHE_$CI_PIPELINE_ID.tar.gz"

  echo "[CI] Gerando cache da build da pasta $OUTPUT/*/target/ ... (Compressing $OUTPUT/*/target/)"
  tar -zcf $CACHE_FILE_NAME $OUTPUT/*/target/
  createS3BuildCache
}

# Execução dos jobs
getopts "bstpxrkdl" job;
case "$job" in

  b) echo "[CI] Executando build, testes e análise do Sonar..."
    python3 senior-ci/common/validations/validate_issues.py

    use_snapshot_version

    VERSION="$CI_COMMIT_REF_SLUG"
    docker_login

    call_extension "before_build"

    generate_sources
    determine_properties
    print_job_vars
    set_version

    versioned_mvn $PARALLEL clean install --batch-mode --errors --update-snapshots -DskipDocker --file $OUTPUT

    generateS3BuildCache

    call_extension "after_build"

    call_extension "before_sonar_preview"
    versioned_mvn $PARALLEL sonar:sonar --batch-mode --errors --projects $SONAR_PROJECTS -Dsonar.host.url=$SONAR_URL -Dsonar.issuesReport.console.enable=true -Dsonar.projectName=$CI_PROJECT_NAME -Dsonar.gitlab.project_id=$CI_PROJECT_ID -Dsonar.analysis.mode=preview -Dsonar.gitlab.commit_sha=$CI_BUILD_REF -Dsonar.gitlab.ref_name=$CI_BUILD_REF_NAME --file $OUTPUT
    call_extension "after_sonar_preview"
    ;;

  s) echo "[CI] Realizando release: SNAPSHOT"
    use_snapshot_version
    publish_snapshot
    ;;

  t) echo "[CI] Realizando release: SNAPSHOT TIMESTAMP"
    use_snapshot_timestamp_version
    publish_snapshot
    ;;

  p) echo "[CI] Publicação da versão..."
    call_extension "before_packaging"
    determine_properties
    VERSION=$CI_COMMIT_TAG

    print_job_vars
    generate_sources
    publish docker

    USER_DEPLOY_MAIL="${GITLAB_USER_EMAIL%@senior.com.br}"
    MESSAGE="O usuário $USER_DEPLOY_MAIL liberou a versão $VERSION do projeto $CI_PROJECT_NAME. Changelog: $CI_PROJECT_URL/blob/$CI_COMMIT_REF_NAME/CHANGELOG.md"

    call_extension "after_packaging"

    notify_text_teams_channel "$MESSAGE"

    echo "[CI] Versão $VERSION publicada no Docker Hub com sucesso!"
    ;;

  x) echo "[CI] Realizando publicação dos resultados no SonarQube - $SONAR_URL"
      VERSION="$CI_COMMIT_REF_SLUG"
      docker_login

      call_extension "before_sonar_send"

      determine_properties
      generate_sources
      set_version

      versioned_mvn $PARALLEL clean compile install --batch-mode --errors --update-snapshots -DskipDocker -Dmaven.test.failure.ignore=true --file $OUTPUT
      versioned_mvn $PARALLEL sonar:sonar --batch-mode --errors --projects $SONAR_PROJECTS -Dsonar.links.scm=$CI_PROJECT_URL -Dsonar.host.url=$SONAR_URL -Dsonar.issuesReport.console.enable=true -Dsonar.gitlab.project_id=$CI_PROJECT_ID --file $OUTPUT

      call_extension "after_sonar_send"
      ;;

  r) echo "[CI] Realizando release: $2"
    python3 senior-ci/common/validations/validate_issues.py

    unshallow_repo

    get_next_version $2

    create_release_branch

    call_extension "before_release"

    call_extension "before_change_changelog"
    update_changelog
    call_extension "after_change_changelog"

    determine_properties
    mvn $PARALLEL clean generate-sources --batch-mode --errors --update-snapshots
    set_version

    create_release_tag

    call_extension "after_release"

    echo "[CI] Criação da release versão $VERSION realizado com sucesso. O job 'package' será executado automaticamente para publicar esta versão no Docker Hub."

    notify_release_teams_channel
    ;;

  k)  echo "[CI] Iniciando deploy no ambiente de desenvolvimento..."
    DEPLOYMENT=${DEPLOYMENT:-$CI_PROJECT_NAME}
    : "${DEPLOYMENT:?Configure a integração com o Kubernetes para utilizar a feature de Continuous Deployment. $HELP_MESSAGE}"

    call_extension "before_deploy"
    CI_COMMIT_TAG="${CI_COMMIT_TAG//-/.}"
    CI_COMMIT_TAG="${CI_COMMIT_TAG:1}"
    VERSION="${CI_COMMIT_TAG:-$CI_COMMIT_REF_SLUG-SNAPSHOT}"

    DEPLOYMENT_DATA=$(kubectl --insecure-skip-tls-verify get deployments $DEPLOYMENT -o jsonpath="{range ..containers[*]}{@.name}{\"=\"}{@.image}{end}{\"\n\"}" | grep "=seniorsistemas/\|=seniorsa/" | head -1)
    CONTAINER=$(echo "$DEPLOYMENT_DATA" | cut -d "=" -f 1)
    IMAGE=$(echo "$DEPLOYMENT_DATA" | cut -d "=" -f 2 | cut -d ":" -f 1)

    echo "[CI] Alterando a imagem do deployment $DEPLOYMENT para $IMAGE:$VERSION no ambiente "
    kubectl --insecure-skip-tls-verify set image deployment/$DEPLOYMENT $CONTAINER="$IMAGE:$VERSION"

    DEPLOY_USERNAME="${GITLAB_USER_EMAIL%@senior.com.br}"
    MESSAGE="O usuário $DEPLOY_USERNAME alterou o deployment $DEPLOYMENT para a versão $VERSION."

    call_extension "after_deploy"

    notify_text_teams_channel "$MESSAGE"

    echo "[CI] Deploy realizado com sucesso!"
    ;;

  d)  echo "[CI] Iniciando deploy no ambiente externo..."
    : "${2:?O segundo parametro é obrigatório, pois indica qual ambiente será feito o deploy - beta ou homologx. $MESSAGE_HELP}"
    : "${BASIC_TOKEN:?Configure a variável BASIC_TOKEN no projeto. $MESSAGE_HELP}"
    : "${NAMESPACE:?Configure a variável NAMESPACE no projeto. $MESSAGE_HELP}"

    PLATAFORM_NAME="$2"

    if [[ "$PLATAFORM_NAME" != "beta" && "$PLATAFORM_NAME" != "homologx" ]]
    then
        echo "[CI] Segundo parametro incorreto! Valores possiveis: beta ou homologx"
        exit 1
    fi

    SERVICE=`cat sdl.properties | sed -n -e 's/generator.app.name=//p'`
    : "${SERVICE:?Adicione no sdl.properties a chave generator.app.name com o nome do serviço. $MESSAGE_HELP}"

    SERVICE=`echo ${SERVICE%%[[:cntrl:]]} | xargs`

    CI_COMMIT_TAG="${CI_COMMIT_TAG//-/.}"
    CI_COMMIT_TAG="${CI_COMMIT_TAG:1}"
    VERSION="${CI_COMMIT_TAG:-$CI_COMMIT_REF_SLUG-SNAPSHOT}"

    call_extension "before_deploy_external"

    echo "[CI] Inicializando deploy da versão $VERSION do serviço $SERVICE no ambiente $PLATAFORM_NAME"

    STATUS_CODE=`curl -w %{http_code} "https://platform-$PLATAFORM_NAME.senior.com.br:30006/setImg?ns=$NAMESPACE&img=seniorsa/$SERVICE&ver=$VERSION" \
      -H "authorization: Basic $BASIC_TOKEN" \
      -o response.txt`

    if [[ $STATUS_CODE -ne 200 ]]
    then
        echo "[CI] Status Code:" $STATUS_CODE
        echo "[CI] Vish, algo deu errado, talvez o serviço esteja indisponivel no momento, tente novamente mais tarde por favor. $MESSAGE_HELP"
        exit 1
    fi

    call_extension "after_deploy_external"

    echo "[CI] Deploy realizado com sucesso!"
    ;;

  l) echo "[CI] Iniciando deploy da arquitetura no ambiente leaf..."
    : "${RANCHER_TOKEN:?Configure a variável RANCHER_TOKEN no projeto. $MESSAGE_HELP}"

      SERVICE=`cat sdl.properties | sed -n -e 's/generator.app.name=//p'`
    : "${SERVICE:?Adicione no sdl.properties a chave generator.app.name com o nome do serviço. $MESSAGE_HELP}"

    SERVICE=`echo ${SERVICE%%[[:cntrl:]]} | xargs`

    echo "[CI] Pegando informações do pod do serviço $SERVICE"

    CURRENT_POD_INFO=`curl -s "$LEAF_RANCHER_BASE_URL:$SERVICE" \
      -X 'GET' \
      -H "Authorization: Bearer $RANCHER_TOKEN" \
      --insecure`

    CURRENT_IMAGE_WITH_VERSION_RAW=`echo $CURRENT_POD_INFO  | jq '.containers[].image' --raw-output`
    CURRENT_IMAGE_WITH_VERSION=${CURRENT_IMAGE_WITH_VERSION_RAW/"/"/"\/"}

    CI_COMMIT_TAG="${CI_COMMIT_TAG//-/.}"
    CI_COMMIT_TAG="${CI_COMMIT_TAG:1}"
    VERSION="${CI_COMMIT_TAG:-$CI_COMMIT_REF_SLUG-SNAPSHOT}"

    IMAGE="seniorsa\/$SERVICE"

    NEW_POD_INFO=`echo $CURRENT_POD_INFO | sed "s/$CURRENT_IMAGE_WITH_VERSION/$IMAGE:$VERSION/g"`

    call_extension "before_deploy_external"

    echo "[CI] Inicializando deploy da versão $VERSION do serviço $SERVICE"

    STATUS_CODE=`curl -sw %{http_code} "$LEAF_RANCHER_BASE_URL:$SERVICE" \
      -X 'PUT' \
      -H 'content-type: application/json' \
      -H "Authorization: Bearer $RANCHER_TOKEN" \
      -o response.txt \
      -d "$NEW_POD_INFO" \
      --compressed \
      --insecure`

    if [[ $STATUS_CODE -ne 200 ]]
    then
        echo "[CI] Status Code:" $STATUS_CODE
        echo "[CI] Vish, algo deu errado, talvez o serviço esteja indisponivel no momento, tente novamente mais tarde por favor. $MESSAGE_HELP"
        exit 1
    fi

    call_extension "after_deploy_external"

    echo "[CI] Deploy realizado com sucesso!"
    ;;
esac

